%*******************************************************************************
% runmod( m ) - run the model for m million years and draw it

function dummy = runmod(m)

global conc concs diffcoeff meshsep sourcerate meshlen count xhvec thvec;

global nruns runtimes runavgs minnam shapenam delfact dispform impm calcmode impscale

ml = meshlen;

% check the timestep on the basis of whether it will be stable ..
% first make sure we catch the maximum temperature in this run interval
maxdcoeff = max( dcoeff(runtimes(nruns)), dcoeff(runtimes(nruns)+m) );

if thvec(1,1) ~= -1,
    for i = 1:length(thvec),
        if (thvec(i, 1)>runtimes(nruns)) & (thvec(i,1)<runtimes(nruns)+m),
            maxdcoeff = max( maxdcoeff, dcoeff(thvec(i,1)) );
        end;
    end;
end;

tstepscale = 0.3;
if delfact == 0, 
    tstepscale = 0.49; 
end;
% for the plane sheet, stability analysis shows a value < 0.5 is required
% for the cylinder and sphere, a value < 0.3 is derived empirically (Jenkin et al. 1994).
tstepsize =tstepscale*meshsep^2/maxdcoeff;

if calcmode == 2,
    tstepsize = tstepsize * impscale;
end;
% the Crank-Nicholson algorithm is unconditionally convergent (at least for the plane sheet
% geometry), but the user is allowed to define the scaling for implicit timesteps

n = ceil(m/tstepsize) + 1;
disp( sprintf( 'Commencing %i time steps ...', n ) );
tstepsize = m/n;
% fine-tune the timestep to fit into the requested run time

oldcl = clock;
for i = 1:n,
    newcl = clock;
    elaps = newcl - oldcl;
    secs = elaps(5)*60 + elaps(6);
    if secs > 60,
        disp( sprintf( '%i time steps still to go ...', n-i+1 ) );
        oldcl = clock;
    end;

    t = runtimes(nruns) + i*tstepsize;

    D1 = dcoeff(t);
% UNPUBL+
%    D2 = dcoeff(t + tstepsize);
% The diffusion coefficient might vary "significantly" within the timestep,
% especially if large Crank-Nicholson timesteps are being used. So, rather than use
% diffcoeff*tstepsize (which is biassed towards the start of the step), instead use
% integral of diffcoeff, based on the approximation that D varies exponentially with time
% over the current timestep. In that case, varying from D1 at time t1 to D2 at time t2
% gives Integral Ddt = (t2-t1) * (D1-D2)/log(D1/D2)
%    if D1 == D2,
% UNPUBL-
    integddt = tstepsize * D1;
% UNPUBL+
%    else
%    integddt = tstepsize * (D1-D2)/log(D1/D2);
%    end;
% UNPUBL-
    
% for a plate,    del squared c = d2c/dx2
% for a cylinder, del squared c = d2c/dr2 + (1/r)dc/dr
% for a sphere,   del squared c = d2c/dr2 + (2/r)dc/dr
% reminder - conc ranges from 1 to ml+1
% so real grain radius (or half-width of slab)is meshlen * meshsep
% These equations are recast in finite-difference form in the following 2 ways

%** START OF METHOD-DEPENDENT CALCULATION
if calcmode == 1,
% fully explicit
    vec = (impm*conc(1:ml)')';
    vec(ml) = vec(ml) + conc(ml+1)*(1+delfact/(2*(ml-1)));
    conc(1:ml) = conc(1:ml) + ...
        (integddt/meshsep^2)*vec + tstepsize * sourcerate;
elseif calcmode == 2,
% Crank-Nicholson
    df = integddt/meshsep^2;
    vec = (impm*conc(1:ml)')';
    vec(ml) = vec(ml) + 2*conc(ml+1)*(1+delfact/(2*(ml-1)));
% note the extra 2 here

    vec2(1:ml) = 2*conc(1:ml) + ...
        df*vec + 2 * tstepsize * sourcerate;
    conc(1:ml) = ( 2*diag(ones(ml,1)) - df*impm )\vec2';
end;
%** END OF METHOD-DEPENDENT CALCULATION

% the boundary condition at edge of grain ..
    conc(ml+1:ml+1) = hist(t, 'X');
    count = count + 1;
end;

nruns = nruns + 1;
runtimes(nruns) = t;
concs(nruns, :) = conc;

% to get the mean argon concentration, which depends on geometry ..
% for slab     = mean( conc )
% for cylinder = (2/a)*mean(conc*radius )     = (2/ml)*mean(conc*(ml*radius/a))
% for sphere = (3/a^2)*mean( conc*radius^2) = (3/ml^2)*mean(conc*(ml*radius/a)^2)
% need to numerically integrate: use Simpson's rule. If y goes from 1 to 2n+1,
% mean( y ) ~= (1/6n) * (y1 + 4y2 + 2y3 + 4y4 + ... + 4y(2n) + y(2n+1))
% where 2n = mesh length

yvals = conc .* (0:ml).^delfact;

runavgs( nruns ) = ( yvals(1) + 4*sum(yvals(2*[1:ml/2])      ) + ...
                                2*sum(yvals(2*[1:(ml/2-1)]+1)) + yvals(ml+1) ) * ...
                   (1+delfact)/(3*ml^(delfact+1));

apptc = hist( (runavgs(nruns)*runtimes(nruns-1) - runavgs(nruns-1)*t)/...
               (runavgs(nruns) - runavgs(nruns-1)), 'T' );
disp( sprintf( ...
  'Now D = %12.7f mm2/Ma, Time step = %6.3f Ma, Apparent Tc = %6.1f deg C', ...
      D1, tstepsize, apptc) );

if dispform > 0, 
    figure(1);
end;
timvec = (0:100)*runtimes(nruns)/100;
    % used only when plotting user-defined T and X histories
for disptype = 1:dispform,

% plot current age profile
    if disptype == 1,
        subplot(2,2,1);
    else
        figure(2);
    end;
    hold on;
    plot( (0:meshlen)*meshsep, conc );
    title( [minnam, shapenam] );
    xlabel( sprintf( 'Distance from centre (mm) [%i nodes]', meshlen+1 ));
    ylabel( 'Apparent age (Ma)' );

% plot excess argon (X) history
    if disptype == 1,
        subplot(2,2,2);
    else
        figure(3);
    end;
    hold off;
    if xhvec(1,1) == -1,
        plot( timvec, userxhst(timvec) );
    else
        for i = 1:length(xhvec),
            if t < xhvec(i,1), i = i - 1; break; end;
        end;
        i = i + 1;
        plot( [xhvec(1:i-1,1); t], [xhvec(1:i-1,2); hist(t,'X')] );
    end;
    hold on;
    for i = 1:nruns,
        plot( runtimes(i), hist(runtimes(i), 'X'), 'r*' );
    end;
    xlabel( 'Time (Ma)' );
    ylabel( 'Apparent age at edge' );

% plot volume average apparent age
    if disptype == 1,
        subplot(2,2,3);
    else
        figure(4);
    end;
    hold off;
    plot( runtimes(1:nruns), runavgs(1:nruns) );
    hold on;
    plot( runtimes(1:nruns), runavgs(1:nruns), 'r*' );

    title( 'Bulk age' );
    xlabel( 'Time (Ma)' );
    ylabel( 'Apparent age (Ma)' );

% plot temperature (T) history
    if disptype == 1,
        subplot(2,2,4);
    else
        figure(5);
    end;
    hold off;
    if thvec(1,1) == -1,
        plot( timvec, userthst(timvec) );
    else
        for i = 1:length(thvec),
            if t < thvec(i,1), i = i-1; break; end;
        end;
        i = i + 1;
        plot( [thvec(1:i-1,1); t], [thvec(1:i-1,2); hist(t,'T')] );
    end;
    hold on;
    for i = 1:nruns,
        plot( runtimes(i), hist(runtimes(i), 'T'), 'r*' );
    end;
    xlabel( 'Time (Ma)' );
    ylabel( 'Temperature (deg C)' );

end;

